@@ -3,14 +3,15 @@
 # Project   : Cuisine - Functions to write Fabric recipes
 # -----------------------------------------------------------------------------
 # License   : Revised BSD License
+# -----------------------------------------------------------------------------
 # Authors   : Sebastien Pierre                            <sebastien@ffctn.com>
 #             Thierry Stiegler   (gentoo port)     <thierry.stiegler@gmail.com>
 #             Jim McCoy (distro checks and rpm port)      <jim.mccoy@gmail.com>
 #             Warren Moore (zypper package)               <warren@wamonite.com>
-#	   Lorenzo Bivens (pkgin package)	<lorenzobivens@gmail.com>
+#             Lorenzo Bivens (pkgin package)          <lorenzobivens@gmail.com>
 # -----------------------------------------------------------------------------
 # Creation  : 26-Apr-2010
-# Last mod  : 20-May-2013
+# Last mod  : 05-Sep-2013
 # -----------------------------------------------------------------------------
 
 """
@@ -43,9 +44,9 @@ See also:
 from __future__ import with_statement
 import base64, hashlib, os, re, string, tempfile, subprocess, types
 import tempfile, functools, StringIO
-import fabric, fabric.api, fabric.operations, fabric.context_managers
+import fabric, fabric.api, fabric.operations, fabric.context_managers, fabric.state
 
-VERSION               = "0.6.4"
+VERSION               = "0.6.5"
 RE_SPACES             = re.compile("[\s\t]+")
 MAC_EOL               = "\n"
 UNIX_EOL              = "\n"
@@ -165,16 +166,20 @@ def run_local(command, sudo=False, shell=True, pty=True, combine_stderr=None):
 	# TODO: Pass the SUDO_PASSWORD variable to the command here
 	if sudo: command = "sudo " + command
 	stderr   = subprocess.STDOUT if combine_stderr else subprocess.PIPE
-	process  = subprocess.Popen(command, shell=shell, stdout=subprocess.PIPE, stderr=stderr)
+	lcwd = fabric.state.env.get('lcwd', None) or None #sets lcwd to None if it bools to false as well
+	process  = subprocess.Popen(command, shell=shell, stdout=subprocess.PIPE, stderr=stderr, cwd=lcwd)
 	out, err = process.communicate()
 	# FIXME: Should stream the output, and only print it if fabric's properties allow it
 	# print out
+	# SEE: http://docs.fabfile.org/en/1.7/api/core/operations.html#fabric.operations.run
 	# Wrap stdout string and add extra status attributes
-	result = fabric.operations._AttributeString(out.rstrip('\n'))
-	result.return_code = process.returncode
-	result.succeeded   = process.returncode == 0
-	result.failed      = not result.succeeded
-	result.stderr      = StringIO.StringIO(err)
+	result              = fabric.operations._AttributeString(out.rstrip('\n'))
+	result.command      = command
+	result.real_command = command
+	result.return_code  = process.returncode
+	result.succeeded    = process.returncode == 0
+	result.failed       = not result.succeeded
+	result.stderr       = StringIO.StringIO(err)
 	return result
 
 def run(*args, **kwargs):
@@ -190,6 +195,14 @@ def run(*args, **kwargs):
 		else:
 			return fabric.api.run(*args, **kwargs)
 
+def cd(*args, **kwargs):
+	"""A wrapper around Fabric's cd to change the local directory if
+	mode is local"""
+	if is_local():
+		return fabric.api.lcd(*args, **kwargs)
+	return fabric.api.cd(*args, **kwargs)
+
+
 def sudo(*args, **kwargs):
 	"""A wrapper to Fabric's run/sudo commands, using the
 	'cuisine.MODE_SUDO' global to tell whether the command should be run as
@@ -359,13 +372,19 @@ def file_local_read(location):
 	f.close()
 	return t
 
-def file_read(location):
-	"""Reads the *remote* file at the given location."""
+def file_read(location, default=None):
+	"""Reads the *remote* file at the given location, if default is not `None`,
+	default will be returned if the file does not exist."""
 	# NOTE: We use base64 here to be sure to preserve the encoding (UNIX/DOC/MAC) of EOLs
+	if default is None:
+		assert file_exists(location), "cuisine.file_read: file does not exists {0}".format(location)
+	elif not file_exists(location):
+		return default
 	with fabric.context_managers.settings(
 		fabric.api.hide('stdout')
 	):
-		return base64.b64decode(run('cat "%s" | openssl base64' % (location)))
+		frame = run('cat "%s" | openssl base64' % (location))
+		return base64.b64decode(frame)
 
 def file_exists(location):
 	"""Tests if there is a *remote* file at the given location."""
@@ -605,7 +624,7 @@ def dir_ensure(location, recursive=False, mode=None, owner=None, group=None):
 	If we are not updating the owner/group then this can be done as a single
 	ssh call, so use that method, otherwise set owner/group after creation."""
 	if not dir_exists(location):
-		run('mkdir %s "%s" && echo OK ; true' % (recursive and "-p" or "", location))
+		run('mkdir %s "%s"' % (recursive and "-p" or "", location))
 	if owner or group or mode:
 		dir_attribs(location, owner=owner, group=group, mode=mode, recursive=recursive)
 
@@ -742,10 +761,10 @@ def package_install_yum(package, update=False):
 def package_ensure_yum(package, update=False):
 	status = run("yum list installed %s ; true" % package)
 	if status.find("No matching Packages") != -1 or status.find(package) == -1:
-		package_install(package, update)
+		package_install_yum(package, update)
 		return False
 	else:
-		if update: package_update(package)
+		if update: package_update_yum(package)
 		return True
 
 def package_clean_yum(package=None):
@@ -1240,6 +1259,31 @@ def group_user_del(group, user):
 				else:
 						sudo("usermod -G '' '%s'" % (user))
 
+def group_remove(group=None, wipe=False):
+    """ Removes the given group, this implies to take members out the group
+    if there are any.  If wipe=True and the group is a primary one,
+    deletes its user as well.
+    """
+    assert group_check(group), "Group does not exist: %s" % (group)
+    members_of_group = run("getent group %s | awk -F':' '{print $4}'" % group)
+    members = members_of_group.split(",")
+    is_primary_group = user_check(name=group)
+
+    if wipe:
+        if len(members_of_group):
+            for user in members:
+                group_user_del(group, user)
+        if is_primary_group:
+            user_remove(group)
+        else:
+            sudo("groupdel %s" % group)
+
+    elif not is_primary_group:
+            if len(members_of_group):
+                for user in members:
+                    group_user_del(group, user)
+            sudo("groupdel %s" % group)
+
 # =============================================================================
 #
 # SSH
@@ -1311,14 +1355,21 @@ def ssh_unauthorize(user, key):
 # =============================================================================
 
 def upstart_ensure(name):
-	"""Ensures that the given upstart service is running, restarting
+	"""Ensures that the given upstart service is running, starting
 	it if necessary."""
 	with fabric.api.settings(warn_only=True):
 		status = sudo("service %s status" % name)
 	if status.failed:
 		sudo("service %s start" % name)
-	else:
-		sudo("service %s restart" % name)
+
+
+def upstart_stop(name):
+	"""Ensures that the given upstart service is stopped."""
+	with fabric.api.settings(warn_only=True):
+		status = sudo("service %s status" % name)
+	if status.succeeded:
+		sudo("service %s stop" % name)
+
 
 # =============================================================================
 #
